@@ -22,22 +22,24 @@ Objects of this class contain the metadata that describes a class in an
 InterMine model.  Each class has a name, parent classes/interfaces and any
 number of attributes, references and collections. 
 
-InterMine class descriptors are sub classes of L<Moose::Meta::Class>, and thus
-L<Class::MOP::Class>. Please refer to these packages for further documentation.
-
-=head1 CLASS METHODS
+InterMine class descriptors are sub classes of L<Moose::Meta::Class>, 
+and thus L<Class::MOP::Class>. Please refer to these packages for further 
+documentation.
 
 =cut
 
 use Moose;
 extends qw/Moose::Meta::Class/;
 with 'InterMine::Model::Role::Descriptor';
+
 use InterMine::TypeLibrary qw(
-    FieldList FieldHash ClassDescriptorList ClassDescriptor BigInt
+    FieldHash ClassDescriptorList ClassDescriptor BigInt
 );
 use MooseX::Types::Moose qw(ArrayRef Str Bool);
 use Moose::Util::TypeConstraints;
-use Scalar::Util qw(refaddr);
+use Scalar::Util qw(refaddr blessed);
+
+=head1 CLASS METHODS
 
 =head2 create( $name | $name, %attributes | $name, \%attributes | \%attributes )
 
@@ -53,31 +55,25 @@ The class constructor inherited from L<Moose::Meta::Class>.
  Function: create a new ClassDescriptor object
  Args    : model   - the InterMine::Model that this class is a part of
            name    - the class name
-           parents - a list of the classes and interfaces that this classes
-                     extends
+           parents - a list of the classes and interfaces that this 
+                     classes extends
 
-In most normal use cases, the typical user should not need to call this method. 
-It is used internally when parsing the model to build up the list of classes.
+In most normal use cases, the typical user should NOT need 
+to call this method. It is used internally when parsing the 
+model to build up the list of classes.
 
 =cut
 
 # Make class a synonym for meta in the instantiated object
 
-override 'create' => sub {
+override create => sub {
     my $class = shift;
     my $ret = super;
-    $ret->add_method("class", sub {my $self = shift; return $self->meta});
-    $ret->add_attribute(
-        objectId => {
-            reader => "getObjectId",
-            predicate => "hasObjectId",
-            isa => Str,
-        }
-    );
+    $ret->superclasses($ret->superclasses(), 'InterMine::Model::Object');
     return $ret;
 };
 
-# and make name a synonym for package here.
+=head1 INSTANCE ATTRIBUTES
 
 =head2 name | package
 
@@ -97,8 +93,7 @@ has '+name' => (
 
 =head2 own_fields
 
-The list of fields that were declared in this class (not inherited from 
-elsewhere), it has the following accessors:
+Fields that belong to this class directly. 
 
 =head3 add_own_field($field)
 
@@ -110,20 +105,22 @@ Get the full list of fields declared in this class.
 
 =cut 
 
-has own_fields => (
-    traits => ['Array'],
-    is	    => 'ro',
-    isa	    => FieldList,
-    default => sub { [] },
-    handles => {
-	add_own_field  => 'push',
-	get_own_fields => 'elements',
-    },
-);
+sub add_own_field {
+    my $self = shift;
+    my $field = shift;
+    $self->set_field($field->name, $field);
+    $field->field_class($self);
+}
+
+sub get_own_fields {
+    my $self = shift;
+    return grep {$_->field_class eq $self} $self->fields;
+}
 
 =head2 fieldhash
 
-The map of fields for this class. It has the following accessors:
+The map of fields for this class, including inherited fields. 
+It has the following accessors:
 
 =head3 set_field($name, $field)
 
@@ -149,21 +146,54 @@ has fieldhash => (
     isa	    => FieldHash,
     default => sub { {} },
     handles => {
-	set_field	  => 'set',
-	get_field_by_name => 'get',
-	fields		  => 'values',
-	valid_field       => 'defined',
+        set_field	      => 'set',
+        get_field_by_name => 'get',
+        fields            => 'values',
+        valid_field       => 'defined',
     },
 );
 
+=head3 attributes
+
+Return the fields that are instances of L<InterMine::Model::Attribute>. 
+This is not to be confused with L<Class::MOP>'s C<get_all_attributes>.
+
+=cut
+
+sub attributes {
+    my $self = shift;
+    return grep {$_->isa('InterMine::Model::Attribute')} $self->fields;
+}
+
+=head3 references
+
+Return all the fields that are instances of L<InterMine::Model::Reference>,
+but not the subclass L<InterMine::Model::Collection>.
+
+=cut
+
+sub references {
+    my $self = shift;
+    return grep {
+        $_->isa('InterMine::Model::Reference') && ! $_->isa('InterMine::Model::Collection')
+        } $self->fields;
+}
+
+=head3 collections
+
+Return all the fields that are instances of L<InterMine::Model::Collection>.
+
+=cut
+
+sub collections {
+    my $self = shift;
+    return grep {$_->isa('InterMine::Model::Collection')} $self->fields;
+}
+
+
 =head2 parents 
 
-The immediate ancestors of this class.
-
- Usage   : @parent_class_names = $cd->parents();
- Function: return a list of the names of the classes/interfaces that this class
-           directly extends
- Args    : none
+The names of the immediate ancestors of this class.
 
 =cut
 
@@ -197,57 +227,86 @@ has parental_class_descriptors => (
     default => sub {
         my $self = shift;
         $self->superclasses($self->parents);
-        return [
-            map {$self->model->get_classdescriptor_by_name($_)} $self->parents
-        ];
+        return [ map {$self->model->get_classdescriptor_by_name($_)} 
+                $self->parents ];
     },
 );
 
-=head2 ancestors
+=head1 INSTANCE METHODS
+
+=head2 new_object
+
+The instantiation method inherited from L<Moose::Meta::Class>.
+You should not normally need to use this directly. Instead call
+the C<make_new> method in L<InterMine::Model>.
+
+=cut
+
+# sanitize input by removing undef attributes from the list
+
+around new_object => sub {
+    my $orig = shift;
+    my $self = shift;
+    my $args = (ref $_[0] eq 'HASH') ? $_[0] : {@_};
+    for my $key (keys %$args) {
+        my $value = $args->{$key};
+        delete $args->{$key} unless (defined $value);
+        # Horrible hacky solution to unnecessary warnings
+        # THIS SHOULD BE DELETED WHEN A BETTER COERCION 
+        # SOLUTION CAN BE FOUND!!!!
+        if (blessed $value and $value->isa('JSON::Boolean')) {
+            $args->{$key} = $$value;
+        }
+    }
+    return $self->$orig($args);
+};
+
+# and make name a synonym for package here.
+
+
+=head2 get_ancestors
 
 The full inheritance list, including all ancestors in the model.
 
 =cut
 
-has ancestors => (
-    reader     => 'get_ancestors',
-    isa	       => ClassDescriptorList,
-    lazy       => 1,
-    auto_deref => 1,
-    default => sub {
-	my $self = shift;
-	my @inheritance_path = ($self,);
-	my @classes = $self->parental_class_descriptors();
-	for my $class (@classes) {
-	    push @inheritance_path, $class->get_ancestors;
-	}
-	return \@inheritance_path;
-    },
-);
-
+# Implemented as a method to avoid memory leaks
+sub get_ancestors {
+    my $self = shift;
+    my @inheritance_path = ($self,);
+    my @classes = $self->parental_class_descriptors();
+    for my $class (@classes) {
+        push @inheritance_path, $class->get_ancestors;
+    }
+    return @inheritance_path;
+}
 
 has package => (
     is      => 'ro',
-    isa     => 'Str',
+    isa     => Str,
     default => sub { shift->name }
 );
 
-=head2 add_field
+=head2 add_field(FieldDescriptor $field, Bool own)
 
- Usage   : $cd->add_field($field);
- Function: add a Field to this class
- Args    : $field - a sub class of InterMine::Model::Field
+Add a field to the class. If there is already a field 
+of the same name, it will not be added twice. Setting the boolean
+flag "own" marks this field as originating in this class
+
+See also: InterMine::Model->_fix_class_descriptors
 
 =cut
 
-# see also: Model->_fix_class_descriptors()
 sub add_field {
   my ($self, $field, $own)  = @_;
 
   return if defined $self->get_field_by_name($field->name);
 
-  $self->set_field($field->name, $field);
-  $self->add_own_field($field) if $own;
+  if ($own) {
+    $self->add_own_field($field);
+  } else {
+    $self->set_field($field->name, $field);
+  }
 }
 
 sub _make_fields_into_attributes {
@@ -268,56 +327,16 @@ sub _make_fields_into_attributes {
     }
 }
 
-=head2 attributes
-
- Usage   : @fields = $cd->attributes();
- Function: Return the Attribute objects for the attributes of this class
- Args    : none
-
-=cut
-
-sub attributes {
-    my $self = shift;
-    return grep {$_->isa('InterMine::Model::Attribute')} $self->fields;
-}
-
-=head2 references
-
- Usage   : @fields = $cd->references();
- Function: Return the Reference objects for the references of this class
- Args    : none
-
-=cut
-
-sub references {
-    my $self = shift;
-    return grep {$_->isa('InterMine::Model::Reference')} $self->fields;
-}
-
-=head2 collections
-
- Usage   : @fields = $cd->collections();
- Function: Return the Collection objects for the collections of this class
- Args    : none
-
-=cut
-
-sub collections {
-    my $self = shift;
-    return grep {$_->isa('InterMine::Model::Collection')} $self->fields;
-}
-
 =head2 sub_class_of
 
  Usage   : if ($class_desc->sub_class_of($other_class_desc)) { ... }
- Function: Returns true if and only if this class is a sub-class of the given
-           class or is the same class
- Args    : $other_class_desc - a ClassDescriptor
+ Function: Returns true if and only if this class is a sub-class 
+           of the given class or is the same class
+ Args    : $other_class_desc - a ClassDescriptor, or name of one
 
 =cut
 
-sub sub_class_of
-{
+sub sub_class_of {
   my $self = shift;
   my $other_class_desc = shift;
 
