@@ -58,6 +58,33 @@ package InterMine::Model::Collection;
 use Moose;
 extends 'InterMine::Model::Reference';
 
+override '_get_moose_type' => sub {
+    my $self = shift;
+    return 'ArrayOf' . super;
+};
+
+override '_get_moose_options' => sub {
+    my $self = shift;
+    my @ops = super;
+    my $push_method = "add" . ucfirst($self->name);
+    my $get_method = "get" . ucfirst($self->name);
+    $push_method =~ s/s$//;
+    $get_method =~ s/s$//;
+    my $handles = {};
+    $handles->{$push_method} = "push";
+
+    # Don't set individual getter if it would conflict with the main one
+    $handles->{$get_method} = "get" unless ("get" . ucfirst($self->name) eq $get_method);
+
+    push @ops, (
+        traits => ['Array'], 
+        auto_deref => 1, 
+        handles => $handles,
+        default => sub { [] },
+    );
+    return @ops;
+};
+
 __PACKAGE__->meta->make_immutable;
 no Moose;
 1;
